'Raymond Allen 2008
'Adaptation of Sparkfun NOKIA Display code for Propeller
'With insight from Paul Baker's NokiaLCD code
'Note there is a couple second delay before the screen activates with this code

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
      ' Epson S1D15G10 Command Set
      DISON       = $af   
      DISOFF      = $ae   
      DISNOR      = $a6  
      DISINV      = $a7  
      COMSCN      = $bb   
      DISCTL      = $ca   
      SLPIN       = $95   
      SLPOUT      = $94   
      PASET       = $75   
      CASET       = $15   
      DATCTL      = $bc   
      RGBSET8     = $ce   
      RAMWR       = $5c   
      RAMRD       = $5d   
      PTLIN       = $a8   
      PTLOUT      = $a9   
      RMWIN       = $e0   
      RMWOUT      = $ee   
      ASCSET      = $aa   
      SCSTART     = $ab   
      OSCON       = $d1   
      OSCOFF      = $d2   
      PWRCTR      = $20   
      VOLCTR      = $81   
      VOLUP       = $d6   
      VOLDOWN     = $d7   
      TMPGRD      = $82   
      EPCTIN      = $cd   
      EPCOUT      = $cc   
      EPMWR       = $fc   
      EPMRD       = $fd   
      EPSRRD1     = $7c   
      EPSRRD2     = $7d   
      NOPS         = $25  'Appended the "S" to the end of NOP so as not to confuse Prop Tool

  #16, CS, SCLK, SDATA, RST         'Pin assignments
      
PUB NokiaTest |i
  
  i:= 0


  DIRA[SCLK]~~  
  DIRA[SDATA]~~
  OUTA[SCLK]~
  OUTA[SDATA]~
  
  'reset display
  DIRA[RST]~~ 
  OUTA[RST]~
  delay_ms(1)
  OUTA[RST]~~
  delay_ms(20)

  
  spi_command(DISCTL)           'display control
  spi_data($03)
  spi_data(32)
  spi_data(12)
  spi_data($00)

  spi_command(COMSCN)  'comscn
  spi_data($01)

  spi_command(OSCON)  'oscon

  spi_command(SLPOUT)  ' sleep out

  spi_command(VOLCTR)   ' electronic volume, this is kinda contrast/brightness
  spi_data(5)           ' this might be different for individual LCDs
  spi_data($01)           

  spi_command(PWRCTR)   ' power ctrl
  spi_data($0f)       'everything on, no external reference resistors
  delay_ms(100)

  spi_command(DISINV)  ' display mode


  spi_command(DATCTL)   ' datctl
  spi_data($00)
  spi_data(0)
  spi_data($01)
  spi_data($00)

  spi_command(RGBSET8)   'setup color lookup table
    ' color table
    'RED
    spi_data(0)
    spi_data(2)
    spi_data(4)
    spi_data(6)
    spi_data(8)
    spi_data(10)
    spi_data(12)
    spi_data(15)
    'GREEN
    spi_data(0)
    spi_data(2)
    spi_data(4)
    spi_data(6)
    spi_data(8)
    spi_data(10)
    spi_data(12)
    spi_data(15)
    'BLUE
    spi_data(0)
    spi_data(4)
    spi_data(9)
    spi_data(15)


  spi_command(NOPS)  'nop

  spi_command(PASET)   ' page start/end ram
  spi_data(2)          ' for some reason starts at 2
  spi_data(131)          
  
  spi_command(CASET)   ' column start/end ram
  spi_data(0)          
  spi_data(131)

  spi_command(RAMWR)      '    write some stuff (background)
  repeat i from 0 to 17999
    spi_data(28)                ' 28 is green
  
  

  spi_command(DISON)            ' display on

  delay_ms(200)

  repeat i from 0 to 159  ' this loop adjusts the contrast, change the number of iterations to get
    spi_command(VOLUP)               ' desired contrast.  This might be different for individual LCDs
    delay_ms(2)


  '   draw a multi-colored square in the center of screen
  repeat i from 0 to 4095
    LCD_put_pixel(i, (i // 64) + 32, (i / 64) + 32)

  repeat
  
PUB LCD_put_pixel(color, x, y)
  x += 2               ' for some reason starts at 2
  spi_command($75)   ' page start/end ram
  spi_data(x)            
  spi_data(132)          
  spi_command($15)   ' column start/end ram
  spi_data(y)            ' for some reason starts at 2
  spi_data(131) 
  spi_command($5C)    ' write some s**t
  spi_data(color) 

PRI SPI_Command(cmd)
  SpiOut(cmd)

PRI SPI_Data(data)
  SpiOut(data|$100)

PRI SPIOUT (Value)| bitNum
'Adapted from Martin Hebel's SHIFTOUT routine from BS2 functions
  outa[SDATA]~             ' Data pin = 0
  dira[SDATA]~~              ' Set data as output
  outa[SCLK]~
  dira[SCLK]~~
  ' Send MSB first               
  REPEAT 9                                                                
     outa[SDATA] := Value >> (9-1)                  ' Set output           
     Value := Value << 1                              ' Shift value right    
     !outa[SCLK]                                      ' cycle clock          
     !outa[SCLK]                                                             
     waitcnt(1000 + cnt)                              ' delay                
  outa[SDATA]~                                            ' Set data to low

PRI delay_ms(milliseconds)
  'pause in milliseconds
  waitcnt(((clkfreq / 1000 * milliseconds - 3932) #> 381) + cnt)       